﻿;{
 ; PBData Version 1.1
 ;
 ; Copyright (c) 2008 by Thomas Schulz / ts-soft@web.de
 ; http://www.realsource.de
 ;
 ; The contents of this file are subject To the Mozilla Public License Version
 ; 1.1 (the "License"); you may not use this file except in compliance with
 ; the License. You may obtain a copy of the License at
 ; http://www.mozilla.org/MPL/
 ;
 ; Software distributed under the License is distributed on an "AS IS" basis,
 ; WITHOUT WARRANTY OF ANY KIND, either express Or implied. See the License
 ; For the specific language governing rights And limitations under the
 ; License.
 ;
 ; The Original Code is PBData_Include.pbi
 ;
 ; The Initial Developer of the Original Code is Thomas Schulz.
 ;
 ; Portions created by Thomas Schulz are Copyright (C) 2008
 ; Thomas Schulz. All Rights Reserved.
;}

CompilerIf Defined(uncompress, #PB_Procedure) = #False
CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Linux
    ImportC #PB_Compiler_Home + "purelibraries/linux/libraries/zlib.a"
  CompilerCase #PB_OS_Windows
    ImportC "zlib.lib"
CompilerEndSelect
  uncompress(*dest, *destLen, *source, sourceLen)
EndImport
CompilerEndIf


Enumeration
  #UDRes_Packer_None
  #UDRes_Packer_jCalg1
  #UDRes_Packer_zip
EndEnumeration

CompilerIf Defined(UDResPack, #PB_Structure) = #False
Structure UDResPack
  Size.q
  CRC.q
  Packer.b
  Magic.b[6]
EndStructure
CompilerEndIf

CompilerIf Defined(_UnpackMemory, #PB_Procedure) = #False
Procedure _UnpackMemory(*SourceMemoryID, SourceLength, *DestionationMemoryID, DestLength, Packer = #UDRes_Packer_jCalg1)
  Select Packer
    Case #UDRes_Packer_zip
      If Not uncompress(*DestionationMemoryID, @DestLength, *SourceMemoryID, SourceLength)
        ProcedureReturn #True
      EndIf
  EndSelect
EndProcedure
CompilerEndIf

Define.l PBData_Index
Global PBData_Count
Restore PBData_CatchInfo
Read.l PBData_Count
Global Dim PBDataArray.l(PBData_Count -1)

For PBData_Index = 1 To PBData_Count
  Read.l PBDataArray(PBData_Index - 1)
Next

Procedure PBData_Get(ResNumber)
  Protected Size.l, CRC.l, Packer, I, MemAdr, MemSize
  Protected *Mem.UDResPack, *DestMem

  If ResNumber < 1 : ProcedureReturn #False : EndIf
  If ResNumber > PBData_Count : ProcedureReturn #False : EndIf
  
  For I = 1 To ResNumber - 1
    MemAdr + PBDataArray(I - 1)
  Next
 
  MemSize = PBDataArray(ResNumber - 1)
  *Mem = AllocateMemory(MemSize)

  If *Mem
    CopyMemory(?PBData_CatchStart + MemAdr, *Mem, MemSize)
    If PeekS(@*Mem\Magic[0], 6, #PB_Ascii) <> "PBDATA"

      ProcedureReturn *Mem
    EndIf

    Size    = *Mem\Size
    CRC     = *Mem\CRC
    Packer  = *Mem\Packer

    If Size > 0
      *DestMEM = AllocateMemory(Size)
      If *DestMEM
        If _UnpackMemory(*Mem + SizeOf(UDResPack), MemorySize(*Mem), *DestMEM, Size, Packer) > 0
          FreeMemory(*Mem)
          If CRC32Fingerprint(*DestMEM, Size) = CRC

            ProcedureReturn *DestMEM
          Else
            FreeMemory(*DestMEM)
          EndIf
        Else
          FreeMemory(*DestMEM)
        EndIf
      EndIf
    Else
      FreeMemory(*Mem)
    EndIf
  EndIf
EndProcedure

Procedure.s PBData_GetFileName(ResNumber)
  Protected *Mem = PBData_Get(PBData_Count)
  Protected FileNames.s
  If *Mem
    FileNames = PeekS(*Mem, -1, #PB_UTF8)
    FreeMemory(*Mem)
    ProcedureReturn StringField(FileNames, ResNumber + 1, ",")
  EndIf
EndProcedure

Procedure PBData_GetResNumber(FileName.s, start = 1)
  Protected *Mem = PBData_Get(PBData_Count)
  Protected FileNames.s
  Protected I
  
  If start < 1 : start = 1 : EndIf
  
  If *Mem
    FileNames.s = PeekS(*Mem, -1, #PB_UTF8)
    FreeMemory(*Mem)
    
    For I = start To PBData_Count
      If UCase(StringField(FileNames, I + 1, ",")) = UCase(FileName)
        ProcedureReturn I
      EndIf
    Next
  EndIf
EndProcedure

Procedure PBData_Save(FileName.s, ResNumber)
  Protected FileID
  Protected *Mem

  FileID = CreateFile(#PB_Any, FileName)
  If FileID
    *Mem = PBData_Get(ResNumber)
    If *Mem
      WriteData(FileID, *Mem, MemorySize(*Mem))
      CloseFile(FileID)
      FreeMemory(*Mem)

      ProcedureReturn #True
    EndIf

    CloseFile(FileID)
    DeleteFile(FileName)
  EndIf
EndProcedure

Procedure PBData_CatchSound(Sound, ResNumber)
  Protected *Mem = PBData_Get(ResNumber)
  Protected Result

  If *Mem
    Result = CatchSound(Sound, *Mem, MemorySize(*Mem))
    FreeMemory(*Mem)

    ProcedureReturn Result
  EndIf

EndProcedure

Procedure PBData_CatchImage(Image, ResNumber, Flag = 0)
  Protected *Mem = PBData_Get(ResNumber)
  Protected Result

  If *Mem
    Result = CatchImage(Image, *Mem, MemorySize(*Mem), Flag)
    FreeMemory(*Mem)

    ProcedureReturn Result
  EndIf

EndProcedure

Procedure PBData_CatchSprite(Sprite, ResNumber, Modus = 0)
  Protected *Mem = PBData_Get(ResNumber)
  Protected Result

  If *Mem
    Result = CatchSprite(Sprite, *Mem, Modus)
    FreeMemory(*Mem)

    ProcedureReturn Result
  EndIf

EndProcedure

Procedure PBData_CatchModule(Module, ResNumber)
  Protected *Mem = PBData_Get(ResNumber)
  Protected Result

  If *Mem
    Result = CatchModule(Module, *Mem, MemorySize(*Mem))
    FreeMemory(*Mem)

    ProcedureReturn Result
  EndIf

EndProcedure

;{ PureDataConverter - BinPack
DataSection
  PBData_CatchStart:
  IncludeBinary "MyPictures.binpack"
  PBData_CatchInfo:
  Data.l 13
  Data.l 6912
  Data.l 293
  Data.l 4226
  Data.l 140
  Data.l 154
  Data.l 73017
  Data.l 17092
  Data.l 3716
  Data.l 4405
  Data.l 40660
  Data.l 24086
  Data.l 14792
  Data.l 159

EndDataSection
;}


; IDE Options = PureBasic 4.30 (Windows - x64)
; CursorPosition = 107
; FirstLine = 76
; Folding = ----
; EnableXP